
package se.bildad.client;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.util.Log;

public class BildadCache extends Activity {
    String external_storage_dir = Environment.getExternalStorageDirectory().getAbsolutePath();
    String storage_dir = external_storage_dir + "/.bildad/";

    public BildadCache() {
    }

    public void cacheBitmap(Bitmap b, String filename) {
        String filepath = storage_dir + filename;

        Log.d("Cache", "Trying to write bitmap to cache as \"" + filepath + "\"");

        boolean exists = (new File(storage_dir)).exists();
        if (!exists) {
            Log.d("Cache", "Creating missing dir path " + storage_dir);
            new File(storage_dir).mkdirs();
        } else {
            Log.d("Cache", "Image cache path exists: " + storage_dir);
        }

        try {
            int bytes = b.getByteCount();
            Log.d("Cache", "Allocating " + bytes + " bytes to cache.");

            File file = new File(storage_dir, filename);
            file.createNewFile();
            FileOutputStream fOut = new FileOutputStream(filepath);

            b.compress(Bitmap.CompressFormat.PNG, 100, fOut);
            fOut.flush();
            fOut.close();

        } catch (Exception e) {
            Log.w("Cache", "Could not write \"" + filepath + "\" to cache: " + e.toString());
            e.printStackTrace();
        }
    }

    public Bitmap getCachedBitmap(String filename) {
        String filepath = storage_dir + filename;
        Bitmap b = null;

        Log.d("Cache", "Trying to fetch \"" + filename + "\" from cache.");

        File file = new File(filepath);

        try {
            b = BitmapFactory.decodeStream(new FileInputStream(file), null, null);
        } catch (Exception e) {
            Log.d("Cache", "Could not get bitmap from cache: " + e.toString());
        }

        Log.d("Cache", "Returning \"" + filename + "\" from cache.");

        return b;
    }
}
