
package se.bildad.client;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.http.AndroidHttpClient;
import android.os.AsyncTask;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;

public class BildadNet extends AsyncTask<Void, Void, Void> {
    public List<PhotoObject> photoList = new ArrayList<PhotoObject>();
    public BildadCache bCache = new BildadCache();

    public interface MyCallbackInterface {
        public void onRequestComplete();
    }

    private MyCallbackInterface mCallback;
    private Context context;

    public BildadNet(MyCallbackInterface callback) {
        mCallback = callback;
        context = (Context) callback;
    }

    @Override
    protected Void doInBackground(Void... params) {
        getRecent();
        return null;
    }

    protected void onPostExecute() {
        // In here, call back to Activity or other listener that things are done
        mCallback.onRequestComplete();
    }

    private void sendMessage(String message, Integer percent, Integer photo_id) {
        Log.d("sender", "Broadcasting message");
        Intent intent = new Intent("bildad-net-event");

        intent.putExtra("message", "Message: " + message);
        intent.putExtra("percent", percent);
        intent.putExtra("photo_id", photo_id);
        LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
    }

    public PhotoObject getPhoto(Integer photo_id) {
        Log.d("Network", "In getPhoto(), argument photo_id: " + photo_id);

        return photoList.get(photo_id);
    }

    public List<PhotoObject> getRecent() {
        JSONObject obj = null;

        Log.d("Network", "In getRecent()");

        DefaultHttpClient defaultClient = new DefaultHttpClient();

        // Setup the get request
        // TODO: Make URL an argument
        HttpGet httpGetRequest = new HttpGet("https://bildad.se/test.json");

        Log.d("JSON", "requesting https://bildad.se/test.json");

        String result = null;

        try {
            HttpResponse httpResponse = defaultClient.execute(httpGetRequest);

            HttpEntity httpEntity = httpResponse.getEntity();

            result = EntityUtils.toString(httpEntity);

            Log.d("Network", "Got JSON result: " + result);

            obj = new JSONObject(result);
        } catch (Exception e) {
            Log.d("Network", "Exception: " + e);
        }

        JSONArray photos = null;

        try {
            photos = obj.getJSONArray("photos");
        } catch (JSONException e) {
            e.printStackTrace();
        }

        Bitmap photoBitmap = null;

        Log.d("Photo", "Length of array: " + photos.length());

        for (int i = 0; i < photos.length(); i++) {
            JSONObject row = null;
            String photo_url = null;
            String name = null;

            // TODO: Validate JSON in some more pretty way...
            try {
                row = photos.getJSONObject(i);
            } catch (JSONException e) {
                e.printStackTrace();
            }

            try {
                photo_url = row.getString("url");
            } catch (JSONException e) {
                e.printStackTrace();
            }

            try {
                name = row.getString("name");
            } catch (JSONException e) {
                e.printStackTrace();
            }

            Log.d("Network", "Downloading " + photo_url + ", " + name);

            photoBitmap = getBitmap(photo_url);

            PhotoObject b = new PhotoObject(row, photoBitmap, name);

            sendMessage((i + 1) + "/" + photos.length() + " done!",
                    (((i + 1) * 100) / photos.length()), i);

            photoList.add(b);
        }

        return photoList;
    }

    // Attempt to fetch bitmap from cache. If it's not previously cached,
    // download it and add it to the cache.
    public Bitmap getBitmap(String url) {
        Bitmap b = null;
        String filename = null;

        // Get filename from url
        String[] filename_parts = url.split("/");
        filename = filename_parts[filename_parts.length - 1];

        // Check cache for bitmap
        b = bCache.getCachedBitmap(filename);

        // Return bitmap if found in cache
        if (b != null) {
            return b;
        }

        Log.d("Cache", "No bitmap named \"" + filename + "\" found in cache, downloading...");

        // Bitmap not found, download it and cache it for future use
        b = downloadBitmap(url);

        // Cache bitmap
        bCache.cacheBitmap(b, filename);

        // Return to user
        return b;
    }

    public Bitmap downloadBitmap(String url) {
        final AndroidHttpClient client = AndroidHttpClient.newInstance("Android");
        final HttpGet getRequest = new HttpGet(url);

        try {
            HttpResponse response = client.execute(getRequest);
            final int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) {
                Log.w("ImageDownloader", "Error " + statusCode + " while retrieving bitmap from "
                        + url);
                return null;
            }

            final HttpEntity entity = response.getEntity();
            if (entity != null) {
                InputStream inputStream = null;
                try {
                    inputStream = entity.getContent();
                    final Bitmap bitmap = BitmapFactory.decodeStream(inputStream);
                    return bitmap;
                } finally {
                    if (inputStream != null) {
                        inputStream.close();
                    }
                    entity.consumeContent();
                }
            }
        } catch (Exception e) {
            // Could provide a more explicit error message for IOException or
            // IllegalStateException
            getRequest.abort();
            Log.w("ImageDownloader", "Error while retrieving bitmap from " + url, e);
        } finally {
            if (client != null) {
                client.close();
            }
        }

        return null;
    }
}
