
package se.bildad.client;

import android.os.Bundle;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;
import android.view.Menu;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.Toast;

import se.bildad.client.R;
import se.bildad.client.BildadNet.MyCallbackInterface;

public class MainActivity extends Activity implements MyCallbackInterface {
    private ProgressBar progressBar;
    BildadNet parser = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scroll);

        progressBar = (ProgressBar) findViewById(R.id.progressBar);

        LocalBroadcastManager.getInstance(this).registerReceiver(mMessageReceiver,
                new IntentFilter("bildad-net-event"));

        // Tell user we are checking for updates
        Toast.makeText(getBaseContext(), "Uppdaterar...", Toast.LENGTH_SHORT).show();

        Log.d("Main", "Starting app...");

        try {
            parser = new BildadNet(this);
            parser.execute();
        } catch (Exception e) {
            Log.w("Main", "Exception in onCreate: " + e);
        }
    }

    private BroadcastReceiver mMessageReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            // Get extra data included in the Intent
            String message = intent.getStringExtra("message");
            Integer percent = intent.getIntExtra("percent", 2);
            Integer photo_id = intent.getIntExtra("photo_id", -1);

            if (percent == 100) {
                // Set to 0 to hide progress bar
                // TODO: Make this more pretty
                progressBar.setProgress(0);
            } else {
                progressBar.setProgress(percent);
            }

            Log.d("receiver", "Got message: " + message);
            Log.d("receiver", "Got percent: " + percent);

            LinearLayout linearLayout1 = (LinearLayout) findViewById(R.id.linearLayout1);

            Bitmap photoBitmap = null;

            try {
                photoBitmap = parser.getPhoto(photo_id).get_bitmap();
            } catch (Exception e) {
                Log.d("Exception when getting bitmap: ", e.toString());
            }

            ImageView newImageView = new ImageView(MainActivity.this);
            newImageView.setImageBitmap(photoBitmap);
            linearLayout1.addView(newImageView);
        }
    };

    @Override
    public void onRequestComplete() {
        // TODO: Remove progressbar here
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    protected void onDestroy() {
        // Unregister since the activity is about to be closed.
        LocalBroadcastManager.getInstance(this).unregisterReceiver(mMessageReceiver);
        super.onDestroy();
    }
}
